#ifndef sync_steppable_h
#define sync_steppable_h


/**
 * Alle synchron bewegten Dinge müssen dieses Interface implementieren.
 *
 * @author Hj. Malthaner
 */
class sync_steppable
{
public:
    /**
     * Methode für Echtzeitfunktionen eines Objekts.
     * @return false wenn Objekt aus der Liste der synchronen
     * Objekte entfernt werden sol
     * @author Hj. Malthaner
     */
    virtual bool sync_step(long delta_t) = 0;

    virtual ~sync_steppable() {}
};

#endif
